﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class BeatDetection : MonoBehaviour {
    private AudioSource m_audioSource;
    private float[] m_spectrum = new float[64];
    private int m_currentClipIndex = 0;
    private float m_trackEndTime = 0;

    public AudioClip[] AudioClips;
	public float Intensity;

	public Text Prompt;

	private Color m_active = new Color(239/255f,170/255f,189/255f);
	private Color m_inactive = new Color(.5f,.5f,.5f,.5f);

	private float m_cooldown = 0;
	private float m_cooldownMax = 0.66f;
	private float m_diff = 0.05f;
	private float m_falloff = 10f;
	public bool SweetSpot = false;

	// Use this for initialization
	void Start ()
    {
		m_cooldown = 0;
        m_audioSource = GetComponent<AudioSource>();
        QueuePlaylist();
    }
	
	// Update is called once per frame
    void Update()
    {
        CheckSongChange();

		m_audioSource.GetSpectrumData(m_spectrum, 0, FFTWindow.BlackmanHarris);
		float current = m_spectrum [0] * (m_audioSource.volume > 0.01f ? m_audioSource.volume : 1);

		m_cooldown -= Time.deltaTime;
		if (m_cooldown < 0) {
			m_cooldown = 0;
		}

		if (m_cooldown < m_cooldownMax - 0.2f) {
			SweetSpot = false;
		}

		if (m_cooldown == 0 && Intensity - current > m_diff) {
			m_cooldown = m_cooldownMax;
			SweetSpot = true;
		}
		if (current >= Intensity) {
			Intensity = current;
		} else {
			Intensity -= Time.deltaTime * m_falloff;
		}
		if (Intensity < 0) {
			Intensity = 0;
		}

		Prompt.color = SweetSpot ? m_active : m_inactive;
    }

    public void PlayNextSong()
    {
        m_currentClipIndex = (m_currentClipIndex + 1) % AudioClips.Length;
        m_audioSource.clip = AudioClips[m_currentClipIndex];
        m_trackEndTime = Time.unscaledTime + m_audioSource.clip.length;
        m_audioSource.Play();
    }

    public void PlayerLastSong()
    {
        m_currentClipIndex = (m_currentClipIndex - 1 + AudioClips.Length) % AudioClips.Length;
        m_audioSource.clip = AudioClips[m_currentClipIndex];
        m_trackEndTime = Time.unscaledTime + m_audioSource.clip.length;
        m_audioSource.Play();
    }

    void LazyShuffleClips()
    {
        for (int i = 0; i < AudioClips.Length; i++)
        {
            AudioClip tmpClip = AudioClips[i];
            int randomIndex = Mathf.FloorToInt(AudioClips.Length * Random.value);
            AudioClips[i] = AudioClips[randomIndex];
            AudioClips[randomIndex] = tmpClip;
        }
    }

    void QueuePlaylist()
    {
        LazyShuffleClips();
        m_audioSource.clip = AudioClips[m_currentClipIndex];
        m_audioSource.Play();
        m_trackEndTime = Time.unscaledTime + m_audioSource.clip.length;
    }

    void CheckSongChange()
    {
        if (m_trackEndTime < Time.unscaledTime)
        {
            m_currentClipIndex = (m_currentClipIndex + 1) % AudioClips.Length;
            m_audioSource.clip = AudioClips[m_currentClipIndex];
            m_trackEndTime = Time.unscaledTime + m_audioSource.clip.length;
            m_audioSource.Play();
        }
    }
}
