﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class WaveGrid : MonoBehaviour {
	public Transform[,] Blocks;
	public int Width = 25;
	[Range(0,10)]
	public float Speed = 1.6f;
	[Range(0,1)]
	public float Height = 0.2f;
	[Range(0,1)]
	public float Phase = 0.45f;
	[Range(0,2.29f)]
	public float Direction = 0.46f;

	public LinkedList<RadialWave> RadialWaves;
	public GameObject RadialWave;
	public Transform Disturbance;

	public Material Material;
	public Mesh Mesh;

	public Camera Camera;
	public Vector3 CameraOffset = new Vector3(5,10,5);
	public Vector3 GridOffset = new Vector3(0,0,0);

	public bool Toggle = false;

	// Use this for initialization
	void Awake () {
		RadialWaves = new LinkedList<RadialWave>();
		Blocks = new Transform[Width, Width];
		for (int x = 0; x < Width; x++) {
			for (int z = 0; z < Width; z++) {
				GameObject obj = new GameObject ();
				MeshRenderer renderer = obj.AddComponent<MeshRenderer> ();
				MeshFilter filter = obj.AddComponent<MeshFilter> ();
				renderer.sharedMaterial = Material;
				filter.sharedMesh = Mesh;
				obj.transform.parent = transform;
				obj.transform.localPosition = new Vector3 (x, 0, z);
				Blocks [x, z] = obj.transform;
			}
		}
	}
	
	// Update is called once per frame
	void Update () {
		CheckWaves ();

		Camera.transform.position = Disturbance.position + GridOffset;
		transform.position = Camera.transform.position - CameraOffset;

		Vector3 offset = new Vector3(transform.position.x % 1, transform.position.y % 1, transform.position.z % 1);


		//Debug
		if (Toggle) {
			SpawnRadialWave (4,50);
			Toggle = false;
		}

		for (int x = 0; x < Width; x++) {
			for (int z = 0; z < Width; z++) {
				Blocks[x,z].localPosition = new Vector3(x * transform.localScale.x, 0, z * transform.localScale.z) + offset;
				Vector3 pos = Blocks[x,z].position;
				float y = Height * Mathf.Sin (
					Speed * Time.time + Phase * Mathf.PI *
					(pos.x * Mathf.Cos (Direction) + pos.z * Mathf.Sin (Direction)));

				foreach(RadialWave wave in RadialWaves) {
					Vector3 diff = (wave.transform.position - transform.position) / transform.lossyScale.x;// - offset;

					y += GuassianDistribution(Blocks[x,z].localPosition.x, diff.x, Blocks[x,z].localPosition.z, diff.z, wave.Radius, wave.Height);
					y -= GuassianDistribution(Blocks[x,z].localPosition.x, diff.x, Blocks[x,z].localPosition.z, diff.z, wave.Radius/2, wave.Height);
				}


				Blocks[x,z].position = new Vector3 (pos.x, y, pos.z);
			}
		}
	}

	private float GuassianDistribution(float x, float x_0, float z, float z_0, float sigmaSquared, float amplitude) {
		return amplitude * Mathf.Exp (-( 
			(Mathf.Pow(x-x_0,2) / (2 * sigmaSquared)) + 
			(Mathf.Pow(z-z_0,2) / (2 * sigmaSquared))));
	}

	private void CheckWaves() {
		LinkedListNode<RadialWave> node = RadialWaves.Last;
		if (node == null) {
			return;
		}
		RadialWave wave = node.Value;
		if (wave != null && wave.Radius > wave.Falloff) {
			RadialWaves.RemoveLast ();
			Destroy (wave.gameObject);
		}
	}

	public void SpawnRadialWave(float startHeight, float speed) {
		GameObject obj = Instantiate<GameObject>(RadialWave);
		obj.transform.position = Disturbance.position;
		RadialWave wave = obj.GetComponent<RadialWave> ();
		wave.StartHeight = startHeight;
		wave.Speed = speed;
		RadialWaves.AddFirst (wave);
	}
}
